
:- lib(clean_module/1).
:- lib(read_terms/2).

%% load_dynamic( File, Mod ).
%% load_dynamic( File, Mod, Opts ).
%
% Assert the predicates in a File as dynamics in module Mod.
% This is a very naive version. We only use it for facts.
% It should be an option in the built-in load_files/2.
%
% Opts:
%  * clean(Clean=true)  cleans the module with clean_module/1 before the assertions,
%  (else use _false_).
%==
% ?- open( testo.pl, write, Out ), portray_clause(Out,p(1)), portray_clause(Out,p(2)), close( Out ).
% ?- load_dynamic( testo.pl , testo ).
% ?- p(X).
% Correct to: "testo:p(X)"? yes
% X = 1 ;
% X = 2. 
%
% ?- predicate_property( testo:p(_), X ).
% X = interpreted ;
% X = visible ;
% X = (dynamic) ;
% X = number_of_clauses(2) ;
% X = number_of_rules(0) ;
%==
% 
% @author nicos angelopoulos
% @version  0.1 2014/9/14
%
load_dynamic( File, Mod ) :-
	load_dynamic( File, Mod, [] ).

load_dynamic( File, Mod, Args ) :-
	append( Args, [clean(true)], Opts ),
	memberchk( clean(Clean), Opts ),
	load_dynamic_clean( Clean, File, Mod ).

load_dynamic_clean( true, File, Mod ) :-
	clean_module( Mod ),
	load_dynamic_clean( false, File, Mod ).
load_dynamic_clean( false, File, Mod ) :-
	read_terms( File, Terms ),
	findall( _,           ( member(T,Terms),
					    ( T = (:- Dir) -> call(Dir), fail; true ),
					    assert( Mod:T )
					  ),
					  	_ ).
