
:- use_module( library(lists) ).          % append/3.
:- use_module( library(system) ).         % system/1.
:- use_module( library(apply_macros) ).   % maplist/3.

:- lib(en_list/2).

:- lib(spcs/1).
:- lib(write_def/3).
:- lib(break_nth/4).
:- lib(is_letter/1).
:- lib(breaks_list/5).
:- lib(atoms_concat/2).
:- lib(to_upper_case/2).
:- lib(list_to_tuple/2).
:- lib(delete_leading/3).

write_help( Help, Defs ) :-
     write_help( Help, Defs, [] ).
write_help( Help, Defs, InOpts ) :-
     write_help_term_to_its_bits( Help, Head, Desc, Opts, Note, Exs, ToDo ), 
     ( Head = f(HeadNm) -> 
          Psfx = '(Files|Opts) :-   ' 
          ;
          ( Head = f1(HeadNm) ->
               Psfx = '(File|Opts) :-   '
               ; 
               HeadNm = Head,
               Psfx = '(Opts) :-   ' 
          )
     ),
     write_help_defaults( HelpDefs ),
     append( InOpts, HelpDefs, All ),
     atom_concat( HeadNm, Psfx, Heading ),
     ( memberchk(no_clear(true),InOpts) -> 
          true
          ;
          system( clear )
     ),
     nl, nl,
     write( Heading ), nl,
     atom_codes( Heading, HdingCs ),
     length( HdingCs, HdingLgt ),
     memberchk( width(Screen), All ),
     memberchk( desc_min_right(MinDesc), All ),
     DescOffset is min(HdingLgt,Screen-MinDesc),
     DescLength is Screen - DescOffset, 
     % ( atom(PrvDesc) -> atom_codes(PrvDesc,Desc); PrvDesc = Desc),
     en_list( Desc, DescList ),
     write_description( DescList, DescOffset, DescLength ),
     nl,
     OptHead = 'Options:',
     Offset is 5,
     LengthOff is Screen - Offset, 
     % atom_codes( OptHead, OptHeadCs ),
     % length( OptHeadCs, OptHeadLength ),
     % Spcs is OptHeadLength - 3,
     Spcs is Offset,
     write( OptHead ), nl,
     memberchk(max_opt_length(MaxNm), All),
     maximum_opt_name( Opts, 0, WrtMaxNm ),
     WrtLim is min(MaxNm, 2 + (20 * WrtMaxNm)),
     write_help_options_list( Opts, Defs, Spcs, Screen, WrtLim ),
     en_list( Note, NoteList ),
     ( NoteList == [] -> 
          true
          ;
          nl, write( 'Notes: ' ), nl,
          write_description( NoteList, Offset, LengthOff )
     ),
     en_list( Exs, ExsList ),
     ( ExsList = [] ->
          true
          ;
          ExsHead = 'Examples:',
          % atom_codes( ExsHead, ExsHeadCs ),
          % length( ExsHeadCs, ExsHdLgt ),
          % ExsOff is ExsHdLgt - 2,
          % ExsFor is Screen - ExsOff,
          % ExsFor is Screen - Offset,
          nl, write( ExsHead ), nl,
          maplist( expand_example_element, ExsList, ExpExsList ),
          write_description( ExpExsList, Offset, 1, LengthOff )
     ),
     en_list( ToDo, ToDoList ),
     ( ToDoList = [] ->
          true
          ;
          ToDoHead = 'ToDo :',
          nl, write( ToDoHead ), nl,
          write_description( ToDoList, Offset, LengthOff )
     ),
     nl, nl, nl.

expand_example_element( u(Upsh), Out ) :-
     atom_concat( 'upsh ', Upsh, Out ).
expand_example_element( u(Name,Upsh), Out ) :-
     atoms_concat( ['upsh ',Name,' ', Upsh], Out ).
expand_example_element( lu(Upsh), Out ) :-
     atom_concat( 'lupsh ', Upsh, Out ).
expand_example_element( lu(Name,Upsh), Out ) :-
     atoms_concat( ['lupsh ',Name,' ', Upsh], Out ).
expand_example_element( Out, Out ).
     

write_help_defaults( Defs ) :-
     Defs = [
               width(W),
               desc_min_right(55),
               max_opt_length(18),
               in_opts_space(3)
            ],
     ( getenv('COLUMNS',Col) -> 
          ( number(Col) -> 
               W is Col
               ; 
               atom_number(Col,W)
          )
          ;
          W = 80 
     ).

write_help_options_list( [], _Defs, _Spcs, _Screen, _NmLim ).
write_help_options_list( [H|T], Defs, Spcs, Screen, NmLim ) :-
     ( H = opt(Name,InArgs,Desc); H = opt(Name,Desc), 
               write_help_name_strip(Name,InArgs) ),
     spcs(Spcs), 
     write_help_name_augment( Name, NameAgm, NameClear ),
     write( NameAgm ), write( '(' ), 
     atom_codes( NameAgm, Codes ),
     length( Codes, Length ),
     en_list( InArgs, Args ),
     write_help_option_args( Args, ArgsList, Length, OptTermWrote ),
     Term =.. [NameClear|ArgsList],
     list_to_tuple( ArgsList, ArgsTuple ),
     write( ')' ), 
     flush_output,
     OptNlEdge is NmLim + Spcs + 2,
     ( OptTermWrote > NmLim -> 
          nl, spcs( OptNlEdge )
          ;
          Fill is OptNlEdge - Spcs - OptTermWrote - 2,
          spcs( Fill )
     ),
     atom_codes( Desc, DescCs ),
     For is Screen - OptNlEdge,
     write_with_space_offset( DescCs, OptNlEdge, For, true, Last ),
     Offset is OptNlEdge + 10,
     Limit is Screen - Last,
     write_def( Term, Defs, [write(ArgsTuple),limit(Limit),offset(Offset)] ),
     nl,
     write_help_options_list( T, Defs, Spcs, Screen, NmLim ).

write_description( Desc, Offset, Length ) :-
     write_description( Desc, Offset, false, Length ).
     
write_description( [], _DescOffset, _I, _DescLength ).
write_description( [H|T], DescOffset, I, DescLength ) :-
     atom_codes( H, HCs ),
     ( integer(I) -> 
          number_codes( I, ICs ),
          append( ICs, ". ", ACs ),
          append( ACs, HCs, WCs  ),
          NxI is I + 1
          ;
          NxI = I,
          WCs = HCs
     ),
     write_with_space_offset( WCs, DescOffset, DescLength ),
     nl, % spcs( DescOffset ),
     write_description( T, DescOffset, NxI, DescLength ).

write_with_space_offset( TextCs, Spcs, For ) :-
     write_with_space_offset( TextCs, Spcs, For, false, _Last ).

write_with_space_offset( TextCs, Spcs, For, NoFstSpcs, Last ) :-
     length( TextCs, Length ),
     Length > For,
     break_nth( For, TextCs, LeftCs, RightCs ),
     !,
     ( NoFstSpcs == true -> true; spcs( Spcs ) ),
     reverse( LeftCs, RevLeftCs ), 
     ( (RightCs=[HRgt|_],is_letter(HRgt)) ->
          breaks_list(RevLeftCs, (\+ is_letter(L)), L, RevShiftCs, RevWriteCs)
          ;
          RevWriteCs = RevLeftCs, RevShiftCs = []
     ),
     reverse( RevShiftCs, PrvShiftCs ),
     delete_leading( PrvShiftCs, 0' , ShiftCs ),
     append( ShiftCs, RightCs, RemCs ),
     reverse( RevWriteCs, WriteCs ),
     atom_codes( Write, WriteCs ),
     write( Write ), nl,
     write_with_space_offset( RemCs, Spcs, For, false, Last ).
write_with_space_offset( TextCs, Spcs, _For, NoFstSpcs, Last ) :-
     ( NoFstSpcs == true -> true; spcs( Spcs ) ),
     atom_codes( Text, TextCs ),
     length( TextCs, Length ),
     Last is Spcs + Length,
     write( Text ).

maximum_opt_name( [], MaxNm, MaxNm ).
maximum_opt_name( [H|T], Curr, MaxNm ) :-
     ( H = opt(Name,_Args,_Desc); H = opt(Name,_DescA) ),
     write_help_name_augment(Name, NameAgm, _),
     atom_codes( NameAgm, Codes ),
     length( Codes, This ),
     ( This > Curr -> 
          Nxt is This
          ;
          Nxt is Curr
     ),
     maximum_opt_name( T, Nxt, MaxNm ).

write_help_option_args( [], [], Wrote, Wrote ).
write_help_option_args( [H|T], [_|TArgs], Length, Wrote ) :-
     ( atom(H) -> atom_codes(H, [HHC|THCs]); H = [HHC|THCs] ),
     to_upper_case( [HHC], [UpHHC] ),
     atom_codes( Atm, [UpHHC|THCs] ),
     write( Atm ), 
     length( THCs, THCLength ),
     ( T == [] ->
          NxLength is Length + THCLength + 1
          ;
          write( ',' ),
          NxLength is Length + THCLength + 2
     ),
     write_help_option_args( T, TArgs, NxLength, Wrote ).

write_help_term_to_its_bits( Help, Head, Desc, Opts, Note, Exs, ToDo ) :-
     Help = help(Head,Desc,Opts),
     !,
     write_help_bits_complete_examples( Exs ),
     Note = [], ToDo = [].
write_help_term_to_its_bits( Help, Head, Desc, Opts, Note, Exs, ToDo ) :-
     Help = help(Head,Desc,Opts,Note),
     !,
     write_help_bits_complete_examples( Exs ),
     ToDo = [].
write_help_term_to_its_bits( Help, Head, Desc, Opts, Note, Exs, ToDo ) :-
     Help = help(Head,Desc,Opts,Note,Exs),
     !,
     ToDo = [].
write_help_term_to_its_bits( Help, Head, Desc, Opts, Note, Exs, ToDo ) :-
     Help = help(Head,Desc,Opts,Note,Exs,ToDo).

write_help_bits_complete_examples( Exs ) :-
     ( current_predicate(usage_example/1) ->
          findall( Ex, usage_example(Ex), Exs )
          ;
          Exs = []
     ).

write_help_name_augment( m(Name), NameAgm, Clear ) :-
     !,
     write_help_name_augment( Name, NameInner, Clear ),
     atom_concat( NameInner, '*', NameAgm ).
write_help_name_augment( r(Name), RName, Name ) :-
     !,
     atom_concat( Name, '+', RName ).
write_help_name_augment( Name, Name, Name ).

write_help_name_strip( m(Name), Stripped ) :-
     !,
     write_help_name_strip( Name, Stripped ).
write_help_name_strip( r(Name), Stripped ) :-
     write_help_name_strip( Name, Stripped ).
write_help_name_strip( Name, Name ).
